import sys
import time

def main():
    n = 1000000
    if len(sys.argv) > 1:
        try:
            n = int(sys.argv[1])
        except ValueError:
            pass

    # Adjust N for string concat as it can be memory heavy
    # If the runner sends small N (e.g. 100), we probably want to scale it up 
    # internally to make the benchmark measurable, OR assume the runner N is 
    # the NUMBER OF STRINGS to concat.
    # Velocicode runner usually treats N as "number of times to run the whole benchmark".
    # BUT for these algorithm benchmarks, N usually means "input size". 
    # Let's check how other benchmarks handle it.
    # Fibonacci: N is the Nth number (e.g. 40).
    # Matrix Mul: N is matrix size (e.g. 512).
    # Primes: N is limit (e.g. 1000000).
    
    # So for String Concat, N should be "Number of concats".
    # Let's default a high number for meaningful work if N is small?
    # No, the runner config usually specifies the input size. 
    # Let's look at config.yaml later. For now, assume arg is N.
    
    parts = []
    s = "velocicode"
    
    start_time = time.time()
    
    for _ in range(n):
        parts.append(s)
        
    result = "".join(parts)
    
    duration = time.time() - start_time
    
    # Prevent optimization
    if len(result) == 0:
        print("Error")

if __name__ == "__main__":
    main()



import sqlite3
import sys
import time
import os

def main():
    n = 10000 # inserts
    if len(sys.argv) > 1:
        try:
            n = int(sys.argv[1])
        except:
            pass

    db_path = "bench.db"
    if os.path.exists(db_path):
        os.remove(db_path)

    conn = sqlite3.connect(db_path)
    # Optimization: synchronous=OFF drastically improves insert speed by reducing fsyncs
    # However, for benchmark we might want default safety?
    # Usually "System & I/O" benchmarks want to test I/O.
    # If we disable sync, it becomes CPU/Memory buffer test until flush.
    # Let's keep defaults but wrap inserts in ONE transaction.
    
    conn.execute("CREATE TABLE test (id INTEGER PRIMARY KEY, value TEXT)")
    
    start = time.time()
    
    # 1. Inserts
    # Wrapped in transaction
    conn.execute("BEGIN TRANSACTION")
    for i in range(n):
        conn.execute("INSERT INTO test (value) VALUES (?)", (f"value-{i}",))
    conn.execute("COMMIT")
    
    # 2. Selects
    # Select all or random? sequential?
    # Listing said "Select 10,000 records".
    # Iterate cursor.
    cursor = conn.execute("SELECT * FROM test")
    count = 0
    for row in cursor:
        count += 1
        
    end = time.time()
    conn.close()
    
    # Cleanup
    if os.path.exists(db_path):
        os.remove(db_path)
        
    if count != n:
        print(f"Error: expected {n} rows, got {count}")

if __name__ == "__main__":
    main()



import asyncio
import sys
import time
import argparse

async def io_task(duration):
    # Simulate I/O waiting
    await asyncio.sleep(duration)

async def main_async(n):
    tasks = []
    # Create N tasks
    # To make it more interesting, let's have them actually do something 
    # slightly distinct or just massive concurrency?
    # Massive concurrency is the goal.
    
    # We want to measure the overhead of scheduling.
    # So we spawn N tasks that wait for a bit.
    io_duration = 0.01 # 10ms wait
    
    # Using 0.01s wait for 10,000 tasks -> minimal total wait if parallel
    for _ in range(n):
        tasks.append(io_task(io_duration))
        
    await asyncio.gather(*tasks)

def main():
    try:
        # Default to larger N for async tests because they are fast
        n = 1000 if len(sys.argv) <= 1 else int(sys.argv[1])
        # If N is small (passed by runner default 5?), we should scale it?
        # Runner passes 'iterations' as loop count, but here N usually means "workload size".
        # Runner logic: run_benchmark(..., iterations=5).
        # It calls the binary.
        # Arguments passed to binary: currently main.py just accepts one arg.
        # User config for 'iterations' usually defaults to 5.
        # The arguments passed to the process are controlled by `runner.py` calling `run_cmd.format(...)`.
        # `runner.py` does NOT pass arguments by default unless configured?
        # Wait, `runner.py` loops `iterations` times and calls `cmd`.
        # `cmd` usually includes `{source}`.
        # Where does `N` (workload size) come from?
        # In `fibonacci`, it's hardcoded or passed via argv if supported?
        # Let's check `fibonacci/python/main.py`.
        pass
    except ValueError:
        n = 1000

    # Actually, the runner loops the EXECUTION.
    # The workload size N is usually internal or passed as arg.
    # For `fibonacci`, it's usually fixed (e.g. 35).
    # For `async_io`, we should probably fix the task count to something significant, e.g., 10,000.
    # And allow argv override.
    
    # If sys.argv[1] is provided, use it as task count.
    
    loop_count = 10000 
    if len(sys.argv) > 1:
        try:
            loop_count = int(sys.argv[1])
        except:
             pass

    start_time = time.time()
    asyncio.run(main_async(loop_count))
    duration = time.time() - start_time
    
    # Output check?
    # No output needed for runner, just time.
    
if __name__ == "__main__":
    main()



import platform
import psutil
import cpuinfo

def get_system_info():
    """
    Returns a dictionary containing system information.
    """
    cpu_info = cpuinfo.get_cpu_info()
    cpu_model = cpu_info.get('brand_raw', 'Unknown CPU')
    arch = platform.machine().lower()
    if arch in ['aarch64', 'arm64']:
        arch = 'ARM64'
    elif arch in ['x86_64', 'amd64']:
        arch = 'x86_64'
    else:
        arch = platform.machine()
    
    ram_bytes = psutil.virtual_memory().total
    ram_gb = ram_bytes / (1024**3)
    
    os_info = f"{platform.system()} {platform.release()}"
    
    return {
        "os": os_info,
        "cpu": cpu_model,
        "arch": arch,
        "ram": f"{ram_gb:.1f} GB"
    }



import json
import datetime

HTML_TEMPLATE = """
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Velocicode Benchmark Report</title>
    <script src="https://cdn.jsdelivr.net/npm/chart.js"></script>
    <style>
        body {{ font-family: 'Segoe UI', Tahoma, Geneva, Verdana, sans-serif; margin: 0; padding: 20px; background: #f4f6f8; }}
        .container {{ max-width: 1000px; margin: 0 auto; background: white; padding: 30px; border-radius: 8px; box-shadow: 0 2px 5px rgba(0,0,0,0.1); }}
        h1 {{ color: #2c3e50; border-bottom: 2px solid #3498db; padding-bottom: 10px; }}
        .meta {{ color: #7f8c8d; font-size: 0.9em; margin-bottom: 30px; }}
        .benchmark-section {{ margin-bottom: 50px; }}
        .chart-container {{ position: relative; height: 300px; width: 100%; }}
        table {{ width: 100%; border-collapse: collapse; margin-top: 20px; }}
        th, td {{ padding: 10px; text-align: left; border-bottom: 1px solid #eee; }}
        th {{ background-color: #f8f9fa; }}
        .rank-1 {{ color: #f1c40f; font-weight: bold; }} /* Gold */
        .rank-2 {{ color: #95a5a6; font-weight: bold; }} /* Silver */
        .rank-3 {{ color: #cd7f32; font-weight: bold; }} /* Bronze */
    </style>
</head>
<body>
    <div class="container">
        <h1>Velocicode Report</h1>
        <div class="meta">Generated on: {date}</div>
        
        <div id="charts"></div>
    </div>

    <script>
        const results = {results_json};

        // Group results by benchmark
        const grouped = {{}};
        results.forEach(r => {{
            if (!grouped[r.benchmark]) grouped[r.benchmark] = [];
            grouped[r.benchmark].push(r);
        }});

        const container = document.getElementById('charts');

        // Color palette for languages
        const colors = {{
            'python': '#3572A5',
            'cpp': '#f34b7d',
            'rust': '#dea584',
            'go': '#00ADD8',
            'javascript': '#f1e05a',
            'java': '#b07219',
            'csharp': '#178600'
        }};

        for (const [benchName, items] of Object.entries(grouped)) {{
            // Sort by mean time
            items.sort((a, b) => a.mean - b.mean);
            
            // Create Section
            const section = document.createElement('div');
            section.className = 'benchmark-section';
            
            const title = document.createElement('h2');
            title.textContent = `Benchmark: ${{benchName}}`;
            section.appendChild(title);
            
            // --- Charts Container (Flex) ---
            const chartsRow = document.createElement('div');
            chartsRow.style.display = 'flex';
            chartsRow.style.gap = '20px';
            chartsRow.style.marginBottom = '20px';
            
            // 1. Time Chart
            const divTime = document.createElement('div');
            divTime.className = 'chart-container';
            const cvsTime = document.createElement('canvas');
            divTime.appendChild(cvsTime);
            chartsRow.appendChild(divTime);

            // 2. Memory Chart
            const divMem = document.createElement('div');
            divMem.className = 'chart-container';
            const cvsMem = document.createElement('canvas');
            divMem.appendChild(cvsMem);
            chartsRow.appendChild(divMem);
            
            section.appendChild(chartsRow);
            
            // --- Table ---
            const table = document.createElement('table');
            table.innerHTML = `
                <thead>
                    <tr>
                        <th>Rank</th>
                        <th>Language</th>
                        <th>Time (s)</th>
                        <th>Relative Speed</th>
                        <th>Max RAM</th>
                    </tr>
                </thead>
                <tbody>
                    ${{items.map((item, index) => {{
                        const relative = (item.mean / items[0].mean).toFixed(2) + 'x';
                        let rankClass = '';
                        let rankIcon = index + 1;
                        if (index === 0) {{ rankIcon = '🥇'; rankClass='rank-1'; }}
                        if (index === 1) {{ rankIcon = '🥈'; rankClass='rank-2'; }}
                        if (index === 2) {{ rankIcon = '🥉'; rankClass='rank-3'; }}
                        
                        const memMB = (item.max_memory / (1024*1024)).toFixed(1) + ' MB';

                        return `<tr>
                            <td class="${{rankClass}}">${{rankIcon}}</td>
                            <td>${{item.language}}</td>
                            <td>${{item.mean.toFixed(4)}}</td>
                            <td>${{relative}}</td>
                            <td>${{memMB}}</td>
                        </tr>`;
                    }}).join('')}}
                </tbody>
            `;
            section.appendChild(table);
            
            container.appendChild(section);
            
            // --- Render Time Chart ---
            // --- Render Time Chart ---
            new Chart(cvsTime, {{
                type: 'bar',
                data: {{
                    labels: items.map(i => i.language),
                    datasets: [{{
                        label: 'Time (s) - Lower is Better',
                        data: items.map(i => i.mean),
                        backgroundColor: items.map(i => colors[i.language] || '#cccccc'),
                        borderColor: '#222',
                        borderWidth: 1
                    }}]
                }},
                options: {{
                    responsive: true,
                    maintainAspectRatio: false,
                    indexAxis: 'y',
                    scales: {{
                        x: {{ beginAtZero: true, title: {{ display: true, text: 'Seconds' }} }}
                    }}
                }}
            }});

            // --- Render Memory Chart ---
            // --- Render Memory Chart ---
            new Chart(cvsMem, {{
                type: 'bar',
                data: {{
                    labels: items.map(i => i.language),
                    datasets: [{{
                        label: 'Peak RAM (MB) - Lower is Better',
                        data: items.map(i => (i.max_memory / (1024*1024)).toFixed(2)),
                        backgroundColor: items.map(i => (colors[i.language] || '#cccccc') + '80'),
                        borderColor: '#222',
                        borderWidth: 1
                    }}]
                }},
                options: {{
                    responsive: true,
                    maintainAspectRatio: false,
                    indexAxis: 'y',
                    scales: {{
                        x: {{ beginAtZero: true, title: {{ display: true, text: 'Megabytes (MB)' }} }}
                    }}
                }}
            }});
        }}
    </script>
</body>
</html>
"""

def generate_html_report(results, output_path):
    report_html = HTML_TEMPLATE.format(
        date=datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
        results_json=json.dumps(results)
    )
    
    with open(output_path, 'w') as f:
        f.write(report_html)



from velocicode.reporter import print_table

def test_print_table_empty(capsys):
    print_table([])
    captured = capsys.readouterr()
    assert "No results to display" in captured.out

def test_print_table_data(capsys):
    data = [{
        'benchmark': 'test_bench',
        'language': 'python',
        'mean': 1.23456,
        'min': 1.0,
        'max': 1.5
    }]
    print_table(data)
    captured = capsys.readouterr()
    assert "test_bench" in captured.out
    assert "python" in captured.out
    assert "1.2346" in captured.out # rounded



import re
import sys
import time
import os

def main():
    n = 1
    if len(sys.argv) > 1:
        try:
            n = int(sys.argv[1])
        except ValueError:
            pass

    # data.txt in ../data.txt
    base_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    data_path = os.path.join(base_dir, "data.txt")
    
    if not os.path.exists(data_path):
        data_path = "data.txt" # fallback
        if not os.path.exists(data_path):
             print("Error: data.txt not found")
             sys.exit(1)

    with open(data_path, "r", encoding="utf-8") as f:
        content = f.read()

    # Pre-compile regex for fairness?
    # Yes, typically we benchmark RE execution not compilation if we are looping
    
    # Phone: XXX-XXX-XXXX
    # Email: [a-z]{8}@example.com
    # Basic patterns to match generator
    phone_re = re.compile(r'\d{3}-\d{3}-\d{4}')
    email_re = re.compile(r'[a-z]{8}@example\.com')

    start_time = time.time()
    
    for _ in range(n):
        # We need to act on the content.
        # String is immutable, so replace returns new string.
        # In loop, we should probably operate on original content each time?
        # Yes, otherwise we replace replacemnts.
        
        temp = phone_re.sub("[PHONE]", content)
        result = email_re.sub("[EMAIL]", temp)
        
    duration = time.time() - start_time
    
    if len(result) == 0:
        print("Error")

if __name__ == "__main__":
    main()



import subprocess
import time
import os
import shutil
import psutil

class BenchmarkRunner:
    def __init__(self, config):
        self.config = config



    def run_benchmark(self, language, source_path, iterations=5, on_status=None):
        def log(msg):
            if on_status:
                on_status(msg)
            else:
                print(msg)

        lang_config = self.config.get('languages', {}).get(language)
        if not lang_config:
            raise ValueError(f"Unknown language: {language}")

        # Prepare output binary path if needed
        bin_dir = os.path.join(os.path.dirname(source_path), 'bin')
        os.makedirs(bin_dir, exist_ok=True)
        bin_name = f"runner_{language}"
        bin_path = os.path.join(bin_dir, bin_name)

        # Compile if necessary
        try:
            compile_cmd_tpl = lang_config.get('compile')
            
            # Special handling for Rust with Cargo
            if language == 'rust' and os.path.exists(os.path.join(os.path.dirname(source_path), 'Cargo.toml')):
                log("Detected Cargo.toml, building with cargo...")
                cwd = os.path.dirname(source_path)
                subprocess.check_call("cargo build --release --quiet", shell=True, cwd=cwd)
                compile_cmd_tpl = None # Handled above
                run_cmd_tpl = "cargo run --release --quiet"
                lang_config = lang_config.copy()
                lang_config['run'] = run_cmd_tpl

            # Special handling for Java with Maven
            if language == 'java' and os.path.exists(os.path.join(os.path.dirname(source_path), 'pom.xml')):
                log("Detected pom.xml, building with Maven...")
                cwd = os.path.dirname(source_path)
                # Compile
                subprocess.check_call("mvn package -DskipTests --quiet", shell=True, cwd=cwd)
                compile_cmd_tpl = None
                
                # Run
                # Find the jar in target/
                target_dir = os.path.join(cwd, "target")
                jar_files = [f for f in os.listdir(target_dir) if f.endswith(".jar") and "original" not in f]
                if jar_files:
                    jar_path = os.path.join("target", jar_files[0])
                    run_cmd_tpl = f"java -jar {jar_path}"
                    lang_config = lang_config.copy()
                    lang_config['run'] = run_cmd_tpl
                else:
                    raise FileNotFoundError("Could not find built JAR file in target/ directory")

            if compile_cmd_tpl:
                cmd = compile_cmd_tpl.format(source=source_path, out=bin_path)
                log(f"Compiling...")
                # Run compile in the source directory for all languages to resolve relative files/includes consistently
                cwd = os.path.dirname(source_path)
                subprocess.check_call(cmd, shell=True, cwd=cwd)
            else:
                # No compilation needed
                pass
        except subprocess.CalledProcessError as e:
            log(f"[red]Compilation failed[/red]: {e}")
            return None

        run_cmd_tpl = lang_config.get('run')
        # If compiled, use bin_path, else use source_path
        cmd = run_cmd_tpl.format(source=source_path, out=bin_path)

        times = []
        max_mems = []

        for i in range(iterations):
            log(f"Running iteration {i+1}/{iterations}...")
            start = time.time()
            try:
                # Always run in the source directory for consistency
                cwd = os.path.dirname(source_path)
                
                process = subprocess.Popen(cmd, shell=True, stdout=subprocess.DEVNULL, cwd=cwd)
                
                # Monitor memory
                peak_mem = 0
                try:
                    p = psutil.Process(process.pid)
                    while process.poll() is None:
                        try:
                            # rss is Resident Set Size (RAM usage)
                            mem_info = p.memory_info()
                            peak_mem = max(peak_mem, mem_info.rss)
                        except (psutil.NoSuchProcess, psutil.AccessDenied):
                            break
                        time.sleep(0.005) # Poll every 5ms
                except (psutil.NoSuchProcess, psutil.AccessDenied):
                    # Process finished too quickly
                    pass
                
                process.wait() # Ensure process is done
                
                if process.returncode != 0:
                    raise subprocess.CalledProcessError(process.returncode, cmd)

                duration = time.time() - start
                times.append(duration)
                max_mems.append(peak_mem)
                
            except subprocess.CalledProcessError as e:
                log(f"[red]Runtime error[/red]: {e}")
                return None

        # simple stats
        # simple stats
        avg_mem_usage = sum(max_mems) / len(max_mems) if max_mems else 0
        return {
            'mean': sum(times) / len(times),
            'min': min(times),
            'max': max(times),
            'count': len(times),
            'avg_memory': avg_mem_usage,
            'max_memory': max(max_mems) if max_mems else 0
        }



import argparse
import yaml
import os
import sys
from glob import glob
import shutil
from .runner import BenchmarkRunner
from .reporter import print_table
from .system_info import get_system_info
from rich.console import Console
from rich.panel import Panel
from rich.prompt import Confirm
from rich.progress import Progress, SpinnerColumn, TextColumn
from rich import print as rprint

console = Console()

BENCHMARK_DESCRIPTIONS = {
    "fibonacci": "Recursion & CPU intensive",
    "matrix_mul": "Floating point operations & Cache locality",
    "quicksort": "Memory allocation & Sorting algorithms",
    "primes": "Prime number calculation (Sieve)",
    "string_concat": "String manipulation & memory allocation",
    "json_parse": "JSON parsing & serialization performance",
    "regex_redact": "Regular expression compilation & matching",
    "async_io": "Asynchronous task scheduling overhead",
    "sqlite_ops": "SQLite database Insert/Select performance",
    "http_server": "HTTP server throughput (Requests/sec)"
}

def check_dependencies(config, verbose=True):
    if verbose:
        rprint(Panel("Checking System Dependencies", style="bold blue"))
    
    languages = config.get('languages', {})
    valid_langs = set()
    missing_langs = set()
    
    for lang, cfg in languages.items():
        # Determine the command to check
        # If 'compile' exists, check the compiler. Else check 'run'.
        cmd_str = cfg.get('compile') or cfg.get('run')
        if not cmd_str:
            if verbose:
                rprint(f"[yellow]WARN[/yellow] {lang}: No command defined")
            continue
            
        # simpler parsing: assume command is the first word
        program = cmd_str.split()[0]
        
        path = shutil.which(program)
        if path:
            if verbose:
                rprint(f"[green]✔[/green] {lang:<12} (Found: [cyan]{program}[/cyan])")
            valid_langs.add(lang)
        else:
            if verbose:
                rprint(f"[red]✘[/red] {lang:<12} (Missing: [bold]{program}[/bold])")
                if lang == 'java' and shutil.which('java'):
                    rprint(f"  [dim]Tip: You have 'java' but not 'javac'. Install the JDK package (e.g. java-latest-openjdk-devel).[/dim]")
            missing_langs.add(lang)
            
    return valid_langs, missing_langs


def get_base_dir():
    return os.path.dirname(os.path.abspath(__file__))

def load_config():
    path = os.path.join(get_base_dir(), "config.yaml")
    if not os.path.exists(path):
        print(f"Config file not found: {path}")
        sys.exit(1)
    with open(path, 'r') as f:
        return yaml.safe_load(f)

def discover_benchmarks():
    base_dir = os.path.join(get_base_dir(), "benchmarks")
    # Structure: benchmarks/<algo>/<lang>/<file>
    # We want to find all <algo> directories
    if not os.path.exists(base_dir):
        return []
    return [d for d in os.listdir(base_dir) if os.path.isdir(os.path.join(base_dir, d))]

def main():
    parser = argparse.ArgumentParser(description="Velocicode - Programming Language Speed Benchmark Tool")
    parser.add_argument("action", choices=["run", "list", "check", "deps"], help="Action to perform")
    parser.add_argument("--filter-algo", help="Filter by specific algorithm name")
    parser.add_argument("--filter-lang", help="Filter by specific languages (comma separated)")
    parser.add_argument("--iter", type=int, default=5, help="Number of iterations per benchmark")
    parser.add_argument("--json", help="Export results to a JSON file")
    parser.add_argument("--html", help="Export results to an interactive HTML report")
    
    args = parser.parse_args()
    
    config = load_config()
    
    if args.action == "check":
        check_dependencies(config)
        return

    if args.action == "deps":
        import platform
        system = platform.system().lower()
        
        rprint(Panel("Dependency Installation Guide", style="bold green"))
        print("\nTo install all necessary compilers and runtimes, run the following command for your OS:\n")
        
        if system == "linux":
            # Check for debian/ubuntu vs fedora/rhel
            # Simple heuristic
            is_debian = os.path.exists("/etc/debian_version")
            if is_debian:
                rprint("[bold cyan]Ubuntu/Debian:[/bold cyan]")
                print("sudo apt update && sudo apt install -y build-essential python3-dev golang rustc nodejs default-jdk dotnet-sdk-8.0")
            else:
                rprint("[bold cyan]Fedora/RHEL:[/bold cyan]")
                print("sudo dnf install -y gcc gcc-c++ python3-devel golang rust nodejs java-latest-openjdk-devel dotnet-sdk-8.0")
                
        elif system == "darwin": # macOS
            rprint("[bold cyan]macOS (Homebrew):[/bold cyan]")
            print("brew install gcc python go rust node java dotnet")
            
        elif system == "windows":
            rprint("[bold cyan]Windows (Winget):[/bold cyan]")
            print("winget install Python.Python Microsoft.DotNet.SDK.8 Oracle.JDK.21 GoLang.Go Rustlang.Rustlang OpenJS.NodeJS")
            
        else:
            rprint(f"[yellow]Unknown OS: {system}[/yellow]")
            print("Please install: Python 3, GCC/Clang, Go, Rust, Node.js, Java JDK, .NET SDK")
            
        rprint("\n[dim]Note: ARM64/Apple Silicon is fully supported.[/dim]")
        return

    runner = BenchmarkRunner(config)
    
    algos = discover_benchmarks()
    if args.filter_algo:
        algos = [a for a in algos if a == args.filter_algo]
        
    target_langs = None
    if args.filter_lang:
        target_langs = args.filter_lang.split(',')

    results = []

    if args.action == "list":
        print("Available benchmarks:")
        max_len = max(len(a) for a in algos) if algos else 0
        for a in algos:
            desc = BENCHMARK_DESCRIPTIONS.get(a, "No description available")
            print(f" - {a:<{max_len}} : {desc}")
        return

    if args.action == "run":
        # 1. Display System Info
        sys_info = get_system_info()
        info_text = f"""
[bold]OS:[/bold]   {sys_info['os']}
[bold]CPU:[/bold]  {sys_info['cpu']}
[bold]Arch:[/bold] {sys_info['arch']}
[bold]RAM:[/bold]  {sys_info['ram']}
"""
        rprint(Panel(info_text.strip(), title="System Information", border_style="green"))

        # 2. Check Dependencies
        valid_langs, missing_langs = check_dependencies(config, verbose=False)
        
        if missing_langs:
            rprint(f"[yellow]Warning: The following languages are missing:[/yellow] [bold]{', '.join(missing_langs)}[/bold]")
            if not Confirm.ask("Do you want to continue running valid languages only?"):
                rprint("[red]Aborted.[/red]")
                return

        base_dir = os.path.join(get_base_dir(), "benchmarks")
        
        # Collect all tasks first
        tasks_to_run = []
        for algo in algos:
            algo_dir = os.path.join(base_dir, algo)
            lang_dirs = [d for d in os.listdir(algo_dir) if os.path.isdir(os.path.join(algo_dir, d)) and d != 'bin']
            
            for lang in lang_dirs:
                if target_langs and lang not in target_langs:
                    continue
                
                # SKIP if missing
                if lang in missing_langs:
                    continue
                
                ext_map = {
                    'python': 'main.py',
                    'cpp': 'main.cpp',
                    'rust': 'main.rs',
                    'go': 'main.go',
                    'javascript': 'main.js',
                    'java': 'Main.java',
                    'csharp': 'Program.cs'
                }
                expected_file = ext_map.get(lang)
                if not expected_file:
                    continue
                full_path = os.path.join(algo_dir, lang, expected_file)
                if not os.path.exists(full_path):
                    continue
                tasks_to_run.append((algo, lang, full_path))

        if not tasks_to_run:
            rprint("[yellow]No benchmarks found to run.[/yellow]")
            return

        with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=True
        ) as progress:
            for algo, lang, full_path in tasks_to_run:
                # Initial task description
                desc_tpl = f"[bold cyan]{algo}[/bold cyan] ([magenta]{lang}[/magenta])"
                task_id = progress.add_task(f"{desc_tpl}: Starting...", total=None)
                
                def update_status(msg):
                    # update progress description
                    progress.update(task_id, description=f"{desc_tpl}: {msg}")
                
                stats = runner.run_benchmark(lang, full_path, iterations=args.iter, on_status=update_status)
                progress.remove_task(task_id)
                
                if stats:
                    results.append({
                        'benchmark': algo,
                        'language': lang,
                        **stats
                    })

        print_table(results)
        
        if args.json:
            import json
            with open(args.json, 'w') as f:
                json.dump(results, f, indent=2)
            rprint(f"[green]JSON results exported to:[/green] {args.json}")
            
        if args.html:
            from .report_generator import generate_html_report
            generate_html_report(results, args.html)
            rprint(f"[green]HTML report generated at:[/green] {args.html}")

def interactive_menu():
    rprint(Panel.fit("[bold cyan]Velocicode[/bold cyan]\n[dim]High-Performance Benchmark CLI[/dim]", title="Welcome", border_style="blue"))
    print("1. Run All Benchmarks")
    print("2. Run Specific Algorithm")
    print("3. Run Specific Language")
    print("4. Check Dependencies")
    print("5. Show Install Commands")
    print("6. Exit")
    
    choice = input("Enter choice: ").strip()
    
    if choice == '1':
        return ["run"]
    elif choice == '2':
        algos = discover_benchmarks()
        print("\n[bold]Available Algorithms:[/bold]")
        for i, a in enumerate(algos):
            print(f"{i+1}. {a}")
        idx = input("Select algorithm number: ").strip()
        try:
            algo = algos[int(idx)-1]
            return ["run", "--filter-algo", algo]
        except (ValueError, IndexError):
            print("Invalid selection.")
            return interactive_menu()
    elif choice == '3':
        lang = input("Enter language (e.g. python, cpp): ").strip()
        return ["run", "--filter-lang", lang]
    elif choice == '4':
        return ["check"]
    elif choice == '5':
        return ["deps"]
    elif choice == '6':
        sys.exit(0)
    else:
        print("Invalid choice.")
        return interactive_menu()

if __name__ == "__main__":
    if len(sys.argv) == 1:
        args = interactive_menu()
        # Hack: modify sys.argv so argparse parses our generated args
        sys.argv = [sys.argv[0]] + args
        main()
    else:
        main()



import sys

def fib(n):
    if n <= 1:
        return n
    return fib(n - 1) + fib(n - 2)

if __name__ == "__main__":
    n = 35
    if len(sys.argv) > 1:
        n = int(sys.argv[1])
    print(fib(n))



import os
from velocicode.main import discover_benchmarks, load_config, check_dependencies

def test_discover_benchmarks():
    algos = discover_benchmarks()
    assert 'fibonacci' in algos
    assert 'matrix_mul' in algos
    assert 'quicksort' in algos
    assert 'primes' in algos

def test_load_config():
    config = load_config()
    assert 'languages' in config
    assert 'python' in config['languages']
    assert 'cpp' in config['languages']

def test_check_dependencies_runs(capsys):
    # This just tests that it runs without error and prints something
    config = load_config()
    check_dependencies(config)
    captured = capsys.readouterr()
    assert "Checking System Dependencies" in captured.out



import sys
import random
import time

sys.setrecursionlimit(2000000)

def quick_sort(arr):
    if len(arr) <= 1:
        return arr
    pivot = arr[len(arr) // 2]
    left = [x for x in arr if x < pivot]
    middle = [x for x in arr if x == pivot]
    right = [x for x in arr if x > pivot]
    return quick_sort(left) + middle + quick_sort(right)

if __name__ == "__main__":
    n = 1000000
    if len(sys.argv) > 1:
        n = int(sys.argv[1])
    
    # Generate random data
    data = [random.randint(0, 1000000) for _ in range(n)]
    
    sorted_data = quick_sort(data)
    # Print first element to prevent optimization
    print(sorted_data[0])



import json
import sys
import time
import os

def main():
    # Iterations usually 1 for this, or small, as parsing 12MB is slow
    n = 1
    if len(sys.argv) > 1:
        try:
            n = int(sys.argv[1])
        except ValueError:
            pass

    # Locale data path
    # Assuming script is run from .../python/
    # data.json is in .../json_parse/data.json
    base_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    data_path = os.path.join(base_dir, "data.json")
    
    if not os.path.exists(data_path):
        # Fallback to current dir if copied
        data_path = "data.json"
        
    if not os.path.exists(data_path):
        print("Error: data.json not found")
        sys.exit(1)

    # Read file into memory string first to benchmark PARSING, not I/O
    with open(data_path, "r") as f:
        json_str = f.read()

    start_time = time.time()
    
    for _ in range(n):
        data = json.loads(json_str)
        
    duration = time.time() - start_time
    
    # Prevent optimization
    if len(data) == 0:
        print("Error")

if __name__ == "__main__":
    main()



import sys

def sieve(n):
    primes = [True] * (n + 1)
    p = 2
    while (p * p <= n):
        if (primes[p] == True):
            for i in range(p * p, n + 1, p):
                primes[i] = False
        p += 1
    
    # Count primes
    count = 0
    for p in range(2, n + 1):
        if primes[p]:
            count += 1
    return count

if __name__ == "__main__":
    n = 1000000
    if len(sys.argv) > 1:
        n = int(sys.argv[1])
    
    print(sieve(n))



import sys
import random

def generate_matrix(n):
    return [[random.random() for _ in range(n)] for _ in range(n)]

def mat_mul(A, B, n):
    C = [[0] * n for _ in range(n)]
    for i in range(n):
        for j in range(n):
            sum_val = 0
            for k in range(n):
                sum_val += A[i][k] * B[k][j]
            C[i][j] = sum_val
    return C

if __name__ == "__main__":
    n = 200
    if len(sys.argv) > 1:
        n = int(sys.argv[1])
    
    A = generate_matrix(n)
    B = generate_matrix(n)
    C = mat_mul(A, B, n)
    print(C[0][0]) # prevent optimization



import http.server
import socketserver
import threading
import time
import http.client
import sys

PORT = 0 # Let OS pick port
Handler = http.server.BaseHTTPRequestHandler

class FastHandler(Handler):
    def log_message(self, format, *args):
        pass # Silence logging
        
    def do_GET(self):
        self.send_response(200)
        self.send_header("Content-type", "text/plain")
        self.send_header("Content-Length", "11")
        self.end_headers()
        self.wfile.write(b"Hello World")

def run_server(server_ready_event, server_socket_info):
    # Create server
    with socketserver.TCPServer(("127.0.0.1", 0), FastHandler) as httpd:
        server_socket_info['port'] = httpd.server_address[1]
        server_ready_event.set()
        try:
            httpd.serve_forever()
        except KeyboardInterrupt:
            pass
        except OSError: # Socket closed
            pass

def run_client(port, n):
    conn = http.client.HTTPConnection("127.0.0.1", port)
    for _ in range(n):
        conn.request("GET", "/")
        resp = conn.getresponse()
        resp.read() # Consume
    conn.close()

def main():
    n = 5000 # Default requests
    if len(sys.argv) > 1:
        try:
            n = int(sys.argv[1])
        except:
            pass
            
    server_ready = threading.Event()
    server_info = {}
    
    server_thread = threading.Thread(target=run_server, args=(server_ready, server_info))
    server_thread.daemon = True
    server_thread.start()
    
    server_ready.wait()
    port = server_info['port']
    
    # Warmup? Maybe.
    
    start = time.time()
    run_client(port, n)
    end = time.time()
    
    duration = end - start
    # print(f"Requests per second: {n / duration:.2f}")

if __name__ == "__main__":
    main()



import json
import random
import string
import os

def generate_random_string(length=10):
    return ''.join(random.choices(string.ascii_letters + string.digits, k=length))

def generate_data(num_records=10000):
    data = []
    for i in range(num_records):
        record = {
            "id": i,
            "name": generate_random_string(20),
            "email": generate_random_string(10) + "@example.com",
            "is_active": random.choice([True, False]),
            "tags": [generate_random_string(5) for _ in range(random.randint(1, 5))],
            "meta": {
                "created_at": "2023-01-01T12:00:00Z",
                "score": random.uniform(0, 100)
            }
        }
        data.append(record)
    return data

def main():
    # Target size: ~2-5 MB
    data = generate_data(50000) 
    
    output_path = os.path.join(os.path.dirname(__file__), "data.json")
    with open(output_path, "w") as f:
        json.dump(data, f)
    
    size_mb = os.path.getsize(output_path) / (1024 * 1024)
    print(f"Generated {output_path} ({size_mb:.2f} MB)")

if __name__ == "__main__":
    main()



from setuptools import setup, find_packages

setup(
    name="velocicode",
    version="0.9.1",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    include_package_data=True,
    install_requires=[
        "pyyaml",
        "rich>=13.0.0",
        "psutil",
        "py-cpuinfo",
    ],
    entry_points={
        "console_scripts": [
            "velocicode=velocicode.main:main",
        ],
    },
)



from rich import print
from rich.table import Table
from rich.console import Console

def print_table(results):
    if not results:
        print("[yellow]No results to display[/yellow]")
        return
    
    console = Console()
    
    # 1. Group by Benchmark
    grouped = {}
    for r in results:
        bench_name = r['benchmark']
        if bench_name not in grouped:
            grouped[bench_name] = []
        grouped[bench_name].append(r)
        
    # 2. Process each group
    for bench_name, group_results in grouped.items():
        # Sort by mean time (fastest first)
        group_results.sort(key=lambda x: x['mean'])
        
        # Determine baseline (fastest)
        if not group_results:
            continue
            
        fastest_time = group_results[0]['mean']
        
        # Create Table
        table = Table(
            title=f"Benchmark: [bold cyan]{bench_name}[/bold cyan]",
            show_header=True,
            header_style="bold white",
            border_style="blue",
            expand=True
        )
        
        table.add_column("Rank", style="bold yellow", justify="center", width=4)
        table.add_column("Language", style="magenta", width=12)
        table.add_column("Time (s)", justify="right")
        table.add_column("Relative", justify="right")
        table.add_column("Min (s)", style="dim", justify="right")
        table.add_column("Max (s)", style="dim", justify="right")
        table.add_column("Max RAM", style="cyan", justify="right")

        for i, r in enumerate(group_results):
            # Rank
            rank_display = str(i + 1)
            if i == 0:
                rank_display = "🥇"
            elif i == 1:
                rank_display = "🥈"
            elif i == 2:
                rank_display = "🥉"
            
            # Relative Speed
            rel_speed = r['mean'] / fastest_time
            rel_str = f"{rel_speed:.2f}x"
            
            # Colorize Relative Speed
            if rel_speed < 1.05:
                rel_style = "[green]"
            elif rel_speed < 5.0:
                rel_style = "[yellow]"
            else:
                rel_style = "[red]"
            
            # Memory Formatting
            mem_bytes = r.get('max_memory', 0)
            if mem_bytes > 1024 * 1024:
                mem_str = f"{mem_bytes / (1024*1024):.1f} MB"
            elif mem_bytes > 1024:
                mem_str = f"{mem_bytes / 1024:.1f} KB"
            else:
                mem_str = f"{mem_bytes} B"
            
            table.add_row(
                rank_display,
                r['language'],
                f"{r['mean']:.4f}",
                f"{rel_style}{rel_str}[/]",
                f"{r['min']:.4f}",
                f"{r['max']:.4f}",
                mem_str
            )
            
        console.print(table)
        console.print("") # Newline between tables



import random
import string
import os

def generate_phone():
    # XXX-XXX-XXXX
    return f"{random.randint(100, 999)}-{random.randint(100, 999)}-{random.randint(1000, 9999)}"

def generate_email():
    return ''.join(random.choices(string.ascii_lowercase, k=8)) + "@example.com"

def generate_text_block():
    sentences = [
        "This is a sample sentence.",
        "Please contact me at EMAIL.",
        "My phone number is PHONE.",
        "Ignore this line.",
        "Another EMAIL is here.",
        "Call PHONE for support."
    ]
    block = []
    for _ in range(100):
        s = random.choice(sentences)
        if "EMAIL" in s:
            s = s.replace("EMAIL", generate_email())
        if "PHONE" in s:
            s = s.replace("PHONE", generate_phone())
        block.append(s)
    return " ".join(block)

def main():
    # Generate ~5MB of text
    output_path = os.path.join(os.path.dirname(__file__), "data.txt")
    
    with open(output_path, "w") as f:
        # 1000 blocks * ~5KB each?
        for _ in range(5000):
            f.write(generate_text_block() + "\n")
            
    size_mb = os.path.getsize(output_path) / (1024 * 1024)
    print(f"Generated {output_path} ({size_mb:.2f} MB)")

if __name__ == "__main__":
    main()
